﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TMPro;
using UnityEngine;
using Utility01;

namespace Torture
{
    public class Shredder : MonoBehaviour
    {
        private PhysicalBehaviour physicalBehaviour;
        public AudioClip[] audioClips = new AudioClip[] { };
        public bool activate = false;
        public DamagableMachineryBehaviour damageBehaviour;
        public Collider2D internalCollider;
        public float maxTemperature = 300f;
        public float speed = 40f;
        public AudioSource rumbleSource;
        public SpriteRenderer buttonRenderer;
        public Color enableColor = new Color(0, 1, 0, 0.2f);
        public Color disableColor = new Color(1, 0, 0, 0.2f);
        public ParticleSystem smoke;
        public ParticleSystem debris;
        public bool recoil;
        private float t;
        public Color[] colors = new Color[]
        {
            Color.red,
            new Color(0.6f, 0.28f, 0),
            new Color(1, 0.372f, 0.121f)
        };
        private void Start()
        {
            var phys = gameObject.GetPhysicalBehaviour();
            phys.TrueInitialMass = 20;
            phys.rigidbody.mass = 20;
            gameObject.transform.localScale = Utility.GetModuleVector(gameObject.transform.localScale);
            gameObject.UpdateOutline();
            physicalBehaviour = gameObject.GetComponent<PhysicalBehaviour>();
            physicalBehaviour.RefreshOutline();
            var damagableMachineryExample = ModAPI.FindSpawnable("Industrial Piston").Prefab.GetComponent<DamagableMachineryBehaviour>().ShallowClone();
            damageBehaviour = gameObject.AddComponent<DamagableMachineryBehaviour>();
            damageBehaviour.BreakDamage = 40;
            damageBehaviour.Waterproof = false;
            damageBehaviour.FirearmDamageMultiplier = 1.1f;
            damageBehaviour.ExplosionDamageMultiplier = 1.3f;
            damageBehaviour.BehavioursToToggle = new MonoBehaviour[] { };
            damageBehaviour.GameObjectsToToggle = new GameObject[] { };
            damageBehaviour.ExplosionPrefab = damagableMachineryExample.ExplosionPrefab;

            var actOnCollide = gameObject.AddComponent<ActOnCollideOwn>();
            actOnCollide.DispatchChance = 1f;
            actOnCollide.ImpactForceThreshold = 400f;
            actOnCollide.debug = true;
            actOnCollide.Actions = (collision) =>
            {
                if (!damageBehaviour.Destroyed)
                {
                    ActStop();
                }
            };
            var colliderHolder = Utility.CreateChildObject("holderCollider", gameObject.transform);
            var collider = colliderHolder.AddComponent<BoxCollider2D>();
            Utility.SetPositionWithOffSet(colliderHolder.transform, gameObject.transform, 0.15f, -0.36f);
            collider.size = new Vector2(2.7f, 1f);
            internalCollider = collider;

            rumbleSource = gameObject.AddComponent<AudioSource>();
            rumbleSource.clip = audioClips[0];
            rumbleSource.loop = true;
            rumbleSource.volume = 0.5f;
            rumbleSource.spatialBlend = 1;
            rumbleSource.minDistance = 10;
            rumbleSource.maxDistance = 100;
            gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
        }
        public void ActStop()
        {
            damageBehaviour.Slice();
            rumbleSource.Stop();
            smoke.Stop();
            activate = false;
            recoil = false;
        }
        private void Use()
        {
            AudioSource.PlayClipAtPoint(audioClips[1], transform.position);
            if (!damageBehaviour.Destroyed)
            {
                if (!activate && !damageBehaviour.Destroyed)
                {
                    rumbleSource.Play();
                    smoke.Play();
                }
                else
                {
                    rumbleSource.Stop();
                    smoke.Stop();
                }
                activate = !activate;
            }
        }
        private void Update()
        {
            var modifier = Mathf.Clamp(physicalBehaviour.charge / 20, 1, 10);
            rumbleSource.volume = 0.5f * modifier;
            if (activate && !damageBehaviour.Destroyed && !recoil)
            {
                foreach (var collision in Utility.InCollider(internalCollider))
                {
                    if (collision.gameObject != gameObject)
                    {
                        if (collision.gameObject.TryGetComponent<PhysicalBehaviour>(out PhysicalBehaviour physicalBehaviour))
                        {
                            Crush(physicalBehaviour);
                        }
                    }
                }
            }
            if (recoil && t >= 0)
            {
                t -= Time.deltaTime * modifier;
                if (t <= 0)
                {
                    recoil = false;
                    t = 0.5f;
                }
            }
            buttonRenderer.color = activate ? enableColor : disableColor;
            buttonRenderer.enabled = !damageBehaviour.Destroyed;
        }
        private void SetColorDebris(Color color)
        {
            var debrisMain = debris.main;
            var startColor = debrisMain.startColor;
            startColor.color = color;
            debrisMain.startColor = startColor;
        }
        private void Crush(PhysicalBehaviour physicalBehaviour)
        {
            if (!recoil)
            {
                var materialName = physicalBehaviour.Properties.name.ToLower();
                if (physicalBehaviour.gameObject.TryGetComponent<DestroyableBehaviour>(out DestroyableBehaviour destroyableBehaviour))
                {
                    destroyableBehaviour.Break();
                }
                if (materialName.Contains("wood"))
                {
                    ModAPI.CreateParticleEffect("WoodHit poolable", physicalBehaviour.transform.position);
                    if (physicalBehaviour.Disintegratable)
                    {
                        physicalBehaviour.Disintegrate();
                    }
                    else
                    {
                        physicalBehaviour.gameObject.Destroy();
                    }
                    SetColorDebris(colors[1]);
                    debris.Play();
                }
                else if (materialName.Contains("human"))
                {
                    try
                    {
                        physicalBehaviour.gameObject.SendMessage("Crush");
                    }
                    catch { }
                    SetColorDebris(colors[0]);
                    debris.Play();
                }
                else if (materialName.Contains("gorse"))
                {
                    try
                    {
                        physicalBehaviour.gameObject.SendMessage("Crush");
                    }
                    catch { }
                    SetColorDebris(Color.green);
                    debris.Play();
                }
                else if (physicalBehaviour.Properties.Softness > 0)
                {
                    rumbleSource.PlayOneShot(audioClips[2], 6);
                    ModAPI.CreateParticleEffect("Flash", physicalBehaviour.transform.position);
                    if (materialName.Contains("hollowplastic"))
                    {
                        SetColorDebris(Color.blue);
                    }
                    else if (materialName.Contains("branch"))
                    {
                        SetColorDebris(colors[1]);
                    }
                    else if (materialName.Contains("pumpkin"))
                    {
                        SetColorDebris(colors[2]);
                    }
                    else
                    {
                        SetColorDebris(UnityEngine.Random.ColorHSV());
                    }
                    debris.Play();
                    if (physicalBehaviour.Disintegratable)
                    {
                        physicalBehaviour.Disintegrate();
                    }
                    else
                    {
                        physicalBehaviour.gameObject.Destroy();
                    }
                }
                else
                {
                    ActStop();
                }
                recoil = true;
            }
        }
    }
}
